--------------------------------------
-- Namespaces
--------------------------------------
local _, core = ...
core.AltVaultCharacter = {}

local AltVault = core.AltVault
local AltVaultCharacter = core.AltVaultCharacter

local MAX_LEVEL = 70

local NUM_STATS_TABS = 4

--------------------------------------
-- Delete Alt Data
--------------------------------------

StaticPopupDialogs["DELETE_ALT_DATA"] = {
    text = "Are you sure you want to delete this Character's Data?",
    button1 = "Yes",
    button2 = "No",
    OnAccept = function()
        AltVaultDeleteAlt()
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3
}

function AltDelete_OnClick(self)
	StaticPopup_Show("DELETE_ALT_DATA")
end

function AltVaultDeleteAlt()
	table.remove(AltVaultDB.characters, CharacterJournal.id)
	if #core.AltVault.filteredCharacters == 1 then
		AltVault:SetDefaultFilter(1)
		AltVaultDB.data.selectedFilter = 1
	else
		AltVault:GetFilteredCharacters()
	end
	AltVault:UpdateViews()
end

function AltDeleteFrame_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOP")
	GameTooltip:SetText("Delete Character Data")
end

--------------------------------------
-- Character Info Section
--------------------------------------

function AltRaceIconFrame_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOP")
	local curAlt = AltVaultDB.characters[CharacterJournal.id]
	local race = curAlt.character.race
	local gender = curAlt.character.sex:gsub("^%l", string.upper)
	GameTooltip:SetText(race.." "..gender)
	CursorUpdate(self)
end

function AltSpecIconFrame_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOP")
	local curAlt = AltVaultDB.characters[CharacterJournal.id]
	local spec = curAlt.character.specName
	local class = curAlt.character.class
	local role = curAlt.character.role
	GameTooltip:SetText(spec.." "..class)
	CursorUpdate(self)
end

function AltXPBarFrame_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOP")
	local curAlt = AltVaultDB.characters[CharacterJournal.id]
	local percent = curAlt.character.xp / curAlt.character.xpMax
	if curAlt.character.level == MAX_LEVEL then
		percent = 1
	end
	percent = tostring(ceil(percent * 100)).."%"
	GameTooltip:SetText(percent)
end

function AltRestedXPIcon_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	local curAlt = AltVaultDB.characters[CharacterJournal.id]
	local restedText = "Alt is rested"
	if curAlt.character.restState ~= nil and curAlt.character.restState == "Rested" and curAlt.character.restedXP ~= nil then
		local restedXP = tostring(BreakUpLargeNumbers(curAlt.character.restedXP))
		restedText = restedXP .. " Rested XP Available"
	end
	GameTooltip:SetText(restedText)
end

function AltVaultInfo_OnShow(self)
	AltVaultCharacter:AltVaultUpdateAltInfo(CharacterJournal.id)
end

function AltVaultCharacter:AltVaultUpdateAltInfo(altId)
	local emblem = AltVaultInfo.AltClassEmblem
    local curAlt = AltVaultDB.characters[altId]
	local altClass = string.lower(string.gsub(curAlt.character.class, "%s+", ""))
	if curAlt.character.classKey then
		altClass = curAlt.character.classKey
	end

	if curAlt.GUID == UnitGUID("player") then
		AltDeleteFrame:Hide()
	else
		AltDeleteFrame:Show()
	end

	local emblemAtlas = "talents-animations-class-"..string.lower(altClass)
	emblem:SetAtlas(emblemAtlas)
	emblem:SetVertexColor(0.9, 0.8, 0.2, 1)

	if curAlt.character.title ~= nil then
		local titleBefore = false
		local titleTrim = curAlt.character.title:gsub("^%s*(.-)%s*$", "%1")
		if #titleTrim < #curAlt.character.title then
			AltVaultInfo.AltInfoName:SetPoint("TOP", AltVaultInfo, "TOP", 0, -35)
			AltVaultInfo.AltInfoTitle:SetPoint("TOP", AltVaultInfo, "TOP", 0, -20)
			AltVaultInfo.AltInfoTitle:SetText(titleTrim)
		else
			AltVaultInfo.AltInfoName:SetPoint("TOP", AltVaultInfo, "TOP", 0, -20)
			AltVaultInfo.AltInfoTitle:SetPoint("TOP", AltVaultInfo, "TOP", 0, -42)
			AltVaultInfo.AltInfoTitle:SetText(curAlt.character.title)
		end
		AltVaultInfo.AltInfoName:SetText(curAlt.character.name)
	else
		AltVaultInfo.AltInfoName:SetPoint("TOP", AltVaultInfo, "TOP", 0, -22)
		AltVaultInfo.AltInfoName:SetText(curAlt.character.name)
		AltVaultInfo.AltInfoTitle:SetText("")
	end
	
	local noSpaceRace = string.gsub(curAlt.character.race, "%s+", "")
	local noSpecialRace = string.gsub(noSpaceRace, "'", "")
	local raceString = noSpecialRace:lower()
	if curAlt.character.raceKey then
		raceString = curAlt.character.raceKey:lower()
	end
	if raceString == 'scourge' then
		raceString = "undead"
	elseif raceString == 'highmountaintauren' then
		raceString = "highmountain"
	elseif raceString == 'lightforgeddraenei' then
		raceString = "lightforged"
	elseif raceString == 'zandalaritroll' then
		raceString = "zandalari"
	end
	local iconAtlas = "raceicon-"..raceString.."-"..curAlt.character.sex
	-- AltRaceIconFrame.altRaceIcon:SetMask(nil)
	AltRaceIconFrame.altRaceIcon:SetAtlas(iconAtlas)
	SetPortraitToTexture(AltSpecIconFrame.altSpecIcon, curAlt.character.specBackground)

	if curAlt.character.role ~= nil then
		AltRoleIconFrame.roleIcon:SetTexCoord(GetTexCoordsForRole(curAlt.character.role))
	else
		-- just in case??
		AltRoleIconFrame.roleIcon:SetTexCoord(GetTexCoordsForRole("DAMAGER"))
	end

	AltItemLevelFrame.AltItemLevel:SetText(curAlt.character.ilvl)
	AltItemLevelFrame.AltItemLevel:SetTextColor(
		curAlt.character.ilvlColor.red, 
		curAlt.character.ilvlColor.green, 
		curAlt.character.ilvlColor.blue
	)

	local xpMax = tostring(BreakUpLargeNumbers(curAlt.character.xpMax))
	local xp = tostring(BreakUpLargeNumbers(curAlt.character.xp))
	local xpRatio = xpMax.." / "..xpMax
	local percent = 1
	local width = 230
	if curAlt.character.level ~= MAX_LEVEL then
		percent = curAlt.character.xp / curAlt.character.xpMax
		xpRatio = 'XP: '..xp.." / "..xpMax
		width = width * percent
	end

	AltXPBarFrame.altXPLevel:SetText(curAlt.character.level)
	AltXPBarFrame.altXPProgressBar:SetWidth(width)
	if curAlt.character.level == MAX_LEVEL then
		AltXPBarFrame.altXPPercent:SetText("Max Level")
	else
		AltXPBarFrame.altXPPercent:SetText(xpRatio)
	end

	if curAlt.character.restState == 'Rested' then
		AltXPBarFrame.altRestedXPIcon:Show()
	else
		AltXPBarFrame.altRestedXPIcon:Hide()
	end

	-- STATS FRAME 1 : GENERAL INFO

	local playedString = AltVault:Played_TimeBreakDown(curAlt.character.played, false)
	StatsFrame1.AltPlayedTime:SetText('Total Time Played: '..playedString)

	local playedAtLevelString = AltVault:Played_TimeBreakDown(curAlt.character.playedAtLevel, false)
	StatsFrame1.AltPlayedTimeAtLevel:SetText('Time Played At Level: '..playedAtLevelString)

	local lastPlayedDate = curAlt.character.lastPlayed
	if lastPlayedDate == nil then
		StatsFrame1.AltLastPlayed:SetText('Last Played: Unknown')
	else
		local lastPlayed = AltVault:GetDaysAgo(lastPlayedDate)
		if lastPlayed == 0 then
			StatsFrame1.AltLastPlayed:SetText('Last Played: Today')
		elseif lastPlayed == 1 then
			StatsFrame1.AltLastPlayed:SetText('Last Played: '..lastPlayed.." day ago")
		elseif lastPlayed > 1 then
			StatsFrame1.AltLastPlayed:SetText('Last Played: '..lastPlayed.." days ago")
		end
	end

	-- STATS FRAME 2 : GUILD INFO

	local guildTabard = StatsFrame2.AltTabard
	local guildName = StatsFrame2.AltGuildInfo.AltGuildName
	local guildRank = StatsFrame2.AltGuildInfo.AltGuildRank
	local guildRep = StatsFrame2.AltGuildInfo.AltGuildRep
	local guildMembers = StatsFrame2.AltGuildInfo.AltGuildMembers
	local guildMurloc = StatsFrame2.AltGuildInfo.AltNoGuildFun

	if curAlt.guild ~= nil and curAlt.guild.name ~= nil then
		guildMurloc:Hide()
		local tabardData = curAlt.guild.tabard
		if tabardData ~= nil then
			guildTabard.background:SetVertexColor(tabardData.bgR, tabardData.bgG, tabardData.bgB)
			guildTabard.border:SetVertexColor(tabardData.borderR, tabardData.borderG, tabardData.borderB)
			guildTabard.emblemLeft:SetTexture(tabardData.emblemTexture)
			guildTabard.emblemLeft:SetVertexColor(tabardData.emblemR, tabardData.emblemG, tabardData.emblemB)
			guildTabard.emblemLeft:Show()
			guildTabard.emblemRight:SetTexture(tabardData.emblemTexture)
			guildTabard.emblemRight:SetVertexColor(tabardData.emblemR, tabardData.emblemG, tabardData.emblemB)
			guildTabard.emblemRight:Show()
			guildTabard.emblemFull:Hide()
		else
			guildTabard.emblemRight:Hide()
			guildTabard.emblemLeft:Hide()
			guildTabard.border:SetVertexColor(0.6, 0.4, 0.2)
			guildTabard.background:SetVertexColor(0.9, 0.7, 0.3)
			guildTabard.emblemFull:Show()
		end

		guildRank:SetText("Guild Rank: "..curAlt.guild.rank)
		guildRank:Show()
		guildRep:SetText("Reputation: "..GetText("FACTION_STANDING_LABEL"..curAlt.guild.standingID, curAlt.character.gender))
		guildRep:Show()
		guildMembers:SetText("Member Count: "..curAlt.guild.members)
		guildMembers:Show()
		guildName:SetText(curAlt.guild.name)
	else
		guildTabard.emblemRight:Hide()
		guildTabard.emblemLeft:Hide()
		guildTabard.border:SetVertexColor(0.6, 0.4, 0.2)
		guildTabard.background:SetVertexColor(0.9, 0.7, 0.3)
		guildTabard.emblemFull:Show()
		guildRank:Hide()
		guildRep:Hide()
		guildMembers:Hide()
		guildName:SetText("No Guild")
		guildMurloc:Show()
	end

	guildTabard:Show()

	-- STATS FRAME 3 : MYTHIC PLUS INFO
	local validData = false
	if curAlt.mplus then
		local expiresMonth = curAlt.mplus.dataExpires.month
		local expiresDay = curAlt.mplus.dataExpires.monthDay
		local expiresYear = curAlt.mplus.dataExpires.year
		validData = AltVault:IsTodayOrLater(expiresMonth, expiresDay, expiresYear)
	end
	
	if curAlt.mplus ~= nil and validData then
		StatsFrame3.AltNoMPlusData:Hide()
		StatsFrame3.AltVaultProgress.AltProgress:SetText(curAlt.mplus.vaultProgress..'/'..curAlt.mplus.vaultThreshold)
		StatsFrame3.AltVaultProgress:Show()

		local scoreColor = C_ChallengeMode.GetDungeonScoreRarityColor(curAlt.mplus.dungeonScore)
		StatsFrame3.AltMPlusRating.AltSeasonScore:SetText(curAlt.mplus.dungeonScore)
		if(scoreColor) then
			StatsFrame3.AltMPlusRating.AltSeasonScore:SetVertexColor(scoreColor.r, scoreColor.g, scoreColor.b);
		end
		StatsFrame3.AltMPlusRating:Show()
		if curAlt.mplus.currentKeystoneLevel ~= nil then

			StatsFrame3.AltKeystone.KeystoneBackground:Hide()

			StatsFrame3.AltKeystone.AltKeystoneLevel:SetText(curAlt.mplus.currentKeystoneLevel)
			StatsFrame3.AltKeystone.AltKeystoneLevel:Show()

			local name, _, _, _, backgroundTexture = C_ChallengeMode.GetMapUIInfo(curAlt.mplus.currentKeystoneMap)
			StatsFrame3.AltKeystone.KeystoneDungeonIcon:SetTexture(backgroundTexture)
			StatsFrame3.AltKeystone.KeystoneDungeonIcon:Show()

			local bestKeyFrame = StatsFrame3.AltKeystone.AltSeasonBest			

		else
			StatsFrame3.AltKeystone.KeystoneBackground:Show()
			
			StatsFrame3.AltKeystone.AltKeystoneLevel:Hide()
			StatsFrame3.AltKeystone.KeystoneDungeonIcon:Hide()
		end
		StatsFrame3.AltKeystone:Show()
		StatsFrame3.BestDivider:Show()

		local mapTable = C_ChallengeMode.GetMapTable()
		for mapIndex, mapID in pairs(mapTable) do
			local mapFrame = _G["ChallengeMap"..mapIndex]
			mapFrame:SetID(mapID)
			mapFrame:SetScript("OnEnter", SeasonBestMap_OnEnter)
			local altMapData = curAlt.mplus.bestScores[mapID]
			if altMapData.score > 0 then
				local scoreColor = C_ChallengeMode.GetSpecificDungeonOverallScoreRarityColor(altMapData.overAllScore)
				mapFrame.HighestLevel:SetText(altMapData.level)
				if scoreColor then
					mapFrame.HighestLevel:SetVertexColor(scoreColor.r, scoreColor.g, scoreColor.b)
				end
				mapFrame.HighestLevel:Show()
				mapFrame.Icon:SetDesaturated(false)
			else
				mapFrame.HighestLevel:Hide()
				mapFrame.Icon:SetDesaturated(true)
			end
			
		end
		StatsFrame3.AltSeasonBest:Show()
	else
		StatsFrame3.BestDivider:Hide()
		StatsFrame3.AltNoMPlusData:Show()
		StatsFrame3.AltVaultProgress:Hide()
		StatsFrame3.AltMPlusRating:Hide()
		StatsFrame3.AltKeystone:Hide()
		StatsFrame3.AltSeasonBest:Hide()
	end

	-- STATS FRAME 4 : PVP INFO

	StatsFrame4.AltPvPHonor.AltPvPHonorBar:SetMinMaxValues(0, curAlt.pvp.maxHonor);
	StatsFrame4.AltPvPHonor.AltPvPHonorBar:SetValue(curAlt.pvp.honor);
	StatsFrame4.AltPvPHonor.AltPvPHonorBar.text:SetFormattedText(HEIRLOOMS_PROGRESS_FORMAT, curAlt.pvp.honor, curAlt.pvp.maxHonor);

	StatsFrame4.AltPvPConquest.AltPvPConquestBar:SetMinMaxValues(0, curAlt.pvp.maxConquest);
	StatsFrame4.AltPvPConquest.AltPvPConquestBar:SetValue(curAlt.pvp.currentConquest);
	StatsFrame4.AltPvPConquest.AltPvPConquestBar.text:SetFormattedText(HEIRLOOMS_PROGRESS_FORMAT, curAlt.pvp.currentConquest, curAlt.pvp.maxConquest);

	StatsFrame4.AltPvPTitle:SetText("Honor Level "..curAlt.pvp.honorLevel)
	AltPvPTierIconFrame.icon:SetTexture(curAlt.pvp.tierIcon)
	StatsFrame4.AltPVPHKCount:SetText(BreakUpLargeNumbers(curAlt.pvp.kills))

end

--------------------------------------
-- Alt Equipped Items Rendering
--------------------------------------

function AltItemSlotButton_OnShow(self)
	local name = self:GetName()
	local slotName = string.upper(strsub(name, 4))
	local id, textureName, checkRelic = GetInventorySlotInfo(slotName)
	local equippedItem = AltVaultDB.characters[CharacterJournal.id]["gear"][slotName]
	self:SetID(id)
	if equippedItem ~= nil then
		local item = Item:CreateFromItemLink(equippedItem)
		item:ContinueOnItemLoad(function()
			local itemName, itemHyperLink, itemRarity, _, _, _, _, _, _, itemTexture = GetItemInfo(equippedItem)
			self:SetItemButtonTexture(itemTexture)
			SetItemButtonQuality(self, itemRarity, id)
			SetItemCraftingQualityOverlay(self, itemHyperLink)
			if self.ProfessionQualityOverlay ~= nil then
				self.ProfessionQualityOverlay:Show()
			end
		end)
	else
		self:SetItemButtonTexture(textureName)
		SetItemButtonQuality(self, 1, id)
		self.backgroundTextureName = textureName
	end
end

function AltItemSlotButton_OnModifiedClick(self, button)
	local name = self:GetName()
	local slotName = string.upper(strsub(name, 4))
	local equippedItem = AltVaultDB.characters[CharacterJournal.id]["gear"][slotName]
	local _, itemHyperLink = GetItemInfo(equippedItem)
	HandleModifiedItemClick(itemHyperLink)
end

--------------------------------------
-- Alt Equipped Items Tooltips
--------------------------------------

function AltItemSlotButton_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	local name = self:GetName()
	local slotName = string.upper(strsub(name, 4))
	local equippedItem = AltVaultDB.characters[CharacterJournal.id]["gear"][slotName]
	if ( equippedItem ) then
		GameTooltip:SetHyperlink(equippedItem)
	else
		local text = _G[strupper(slotName)]
		GameTooltip:SetText(text)
	end
	CursorUpdate(self)
end


--------------------------------------
-- Stats window
--------------------------------------

function AltVaultStats_OnShow(self)
	AltVaultCharacter:AltVaultUpdateAltStats(CharacterJournal.id)

	for i = 1, NUM_STATS_TABS do
		local tab = _G["StatsFrame"..i];
		if(AltVaultStatsFrame.selectedStatsTab == i) then
			tab:Show()
		else
			tab:Hide()
		end
	end
end

function AltVaultCharacter:AltVaultUpdateAltStats(altId)
	local curAlt = AltVaultDB.characters[altId]
	local faction = curAlt.character.faction
	if faction ~= "Neutral" then
		local logoTexture = "Interface\\Timer\\"..faction.."-Logo"
		StatsFrame4.AltStatsFactionLogo:SetTexture(logoTexture)
		StatsFrame4.AltStatsFactionLogo:SetVertexColor(0.5, 0.5, 0.5, 0.8)
		StatsFrame4.AltStatsFactionLogo:Show()

		local iconTexture = "Interface\\ICONS\\INV_BannerPVP_0"
		if(faction == 'Alliance') then
			iconTexture = iconTexture..'2'
		elseif(faction == 'Horde') then
			iconTexture = iconTexture..'1'
		end
		StatsSideTab4:SetNormalTexture(iconTexture)
	else
		StatsFrame4.AltStatsFactionLogo:Hide()
	end
end

function AltPvPTierIconFrame_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOP")
	local curAlt = AltVaultDB.characters[CharacterJournal.id]
	local honorText = "Honor: "..curAlt.pvp.honor..'/'..curAlt.pvp.maxHonor
	if curAlt.character.level == MAX_LEVEL then
		honorText = honorText.."\nConquest: "..curAlt.pvp.currentConquest..'/'..curAlt.pvp.maxConquest
	end
	GameTooltip:SetText(honorText)
end

function AltKeystone_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOP")
	local curAlt = AltVaultDB.characters[CharacterJournal.id]

	local keystoneLink = nil
	local bags = curAlt.bags
	for key, bag in pairs(bags) do
		for itemKey, item in pairs(bag.items) do
			if item.link and item.link:find("Keystone:") then
				keystoneLink = item.link
			end
		end
	end 

	if ( keystoneLink ) then
		GameTooltip:SetHyperlink(keystoneLink)
	else
		GameTooltip:SetText("No Keystone")
	end
	CursorUpdate(self)
end

function SeasonBestMap_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOP")
	local curAlt = AltVaultDB.characters[CharacterJournal.id]
	if not curAlt.mplus then return end

	local mapData = curAlt.mplus.bestScores[self:GetID()]
	GameTooltip:SetText(mapData.name, 1, 1, 1)
	if mapData.overAllScore then
		local scoreColor = C_ChallengeMode.GetSpecificDungeonOverallScoreRarityColor(mapData.overAllScore);
		if(not scoreColor) then
			scoreColor = HIGHLIGHT_FONT_COLOR;
		end
		GameTooltip_AddNormalLine(GameTooltip, DUNGEON_SCORE_TOTAL_SCORE:format(scoreColor:WrapTextInColorCode(mapData.overAllScore)), GREEN_FONT_COLOR);
	end

	GameTooltip:Show()
end

--------------------------------------
-- Stats Side Tabs
--------------------------------------

function StatSideTab_OnShow(self)
	local id = self:GetID()
	if(id == 1) then
		self:SetNormalTexture("Interface\\ICONS\\Achievement_General");
	elseif(id == 2) then
		self:SetNormalTexture("Interface\\ICONS\\INV_Shirt_GuildTabard_01");
	elseif(id == 3) then
		self:SetNormalTexture("Interface\\ICONS\\INV_Relics_Hourglass_02");
	elseif(id == 4) then
		local curAlt = AltVaultDB.characters[CharacterJournal.id]
		local faction = curAlt.character.faction
		if(faction == "Alliance") then
			self:SetNormalTexture("Interface\\ICONS\\INV_BannerPVP_02");
		elseif(faction == "Horde") then
			self:SetNormalTexture("Interface\\ICONS\\INV_BannerPVP_01");
		else
			self:SetNormalTexture("Interface\\ICONS\\Achievement_WorldEvent_Brewmaster");
		end
	end

	if(AltVaultStatsFrame.selectedStatsTab == id) then
		self:SetChecked(true)
	else
		self:SetChecked(false)
	end
end

function StatSideTab_OnClick(self)
	PlaySound(SOUNDKIT.IG_CHARACTER_INFO_TAB);
	local id = self:GetID()
	AltVaultStatsFrame.selectedStatsTab = id;

	for i = 1, NUM_STATS_TABS do
		local button = _G["StatsSideTab"..i];
		local tab = _G["StatsFrame"..i];
		if(id == i) then
			button:SetChecked(true)
			tab:Show()
		else
			button:SetChecked(false)
			tab:Hide()
		end
	end
end

function StatSideTab_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOP")
	local tabText = "General Info"
	local id = self:GetID()
	if(id == 2) then
		tabText = "Guild Info"
	elseif(id == 3) then
		tabText = "Mythic Plus"
	elseif(id == 4) then
		tabText = "PVP"
	end
	GameTooltip:SetText(tabText)
end